/*
 *  Copyright © 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import UIKit

@objc class BeaconGatewaySampleView: UIViewController, UITextViewDelegate {
    
    var selectedBeacon:Beacon?
    var beaconGateway:BeaconGatewaySample?
    
    @IBOutlet weak var beaconTypeLabelOutlet: UILabel!
    @IBOutlet weak var currentValueLabelOutlet: UILabel!
    @IBOutlet weak var outputLabelOutlet: UILabel!
    @IBOutlet weak var outputScrollViewOutlet: UIScrollView!
    @IBOutlet weak var outputTextViewOutlet: UITextView!
    @IBOutlet var ViewControllerOutlet: UIView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.title = NAVBARTITLE
    }
    
    var updateTimer:Timer?
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        outputTextViewOutlet.isScrollEnabled = true
        outputTextViewOutlet.isEditable = false
        currentValueLabelOutlet.textAlignment = .left
        currentValueLabelOutlet.numberOfLines = 0
        if updateTimer == nil {
            updateTimer = Timer.scheduledTimer(timeInterval: 5.0, target: self,
                                          selector: #selector(BeaconGatewaySampleView.update),
                                          userInfo: nil, repeats: true)
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        selectedBeacon?.stop()
        updateTimer?.invalidate()
        updateTimer = nil
        super.viewWillDisappear(animated)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @objc func update() {
        
        guard let selectedBeacon = selectedBeacon else {
            return
        }
        let beaconType = selectedBeacon.getType()
        let attrsNormal = [NSAttributedString.Key.font : UIFont.systemFont(ofSize: 15)]
        switch beaconType {
        case .IBEACON:
            self.beaconTypeLabelOutlet.text = "IBeacon"
            let beacon:IBeacon = selectedBeacon as! IBeacon
            
            let boldIBeaconText = "UUID: " + beacon.getUUID() + "\n"
            let attributedString = NSMutableAttributedString(string: boldIBeaconText,
                                                             attributes: attrsNormal)
            let mText = "Major: " + beacon.getMajorVersion() +
                ", Minor: " + beacon.getMinorVersion() + "\n"
            
            let normalString3 = NSMutableAttributedString(string: mText,
                                                          attributes: attrsNormal)
            var normalText = "Signal Strength Measured: " +
                beacon.getMssi().description + ", " + "Received: "
            var distLine = "Distance: "

            let rssi = beaconGateway!.getIBeaconData(beacon: beacon)
            if rssi != 0 {
                normalText += rssi.description + " \n"
                distLine += String(format: "%.3f m\n",
                                   beacon.getDistance())
            } else {
                normalText += "Unknown \n"
                distLine += "Unknown \n"
            }
            
            let normalString = NSMutableAttributedString(string: normalText,
                                                         attributes: attrsNormal)
            let normalString2 = NSMutableAttributedString(string: distLine,
                                                          attributes: attrsNormal)
            
            attributedString.append(normalString3)
            attributedString.append(normalString)
            attributedString.append(normalString2)
            
            currentValueLabelOutlet.attributedText = attributedString
            break
            
        case .EDDYSTONE, .EDDYSTONE_TLM:
            self.beaconTypeLabelOutlet.text = "Eddystone Beacon"
            let beacon:EddystoneBeacon = selectedBeacon as! EddystoneBeacon
            
            // Return the eddystone data, as tuple (rssi,temperature,voltage)
            let data:(rssi:Int?, temperature:Double?, voltage:Double?) =
                beaconGateway!.getEddystoneData(beacon: beacon)
            
            let boldEddyBeaconText = "Namespace: " + beacon.getUUID() + "\n"
            let attributedString = NSMutableAttributedString(string: boldEddyBeaconText,
                                                             attributes: attrsNormal)
            let instText = "Instance: " + beacon.getSerialNumber() + "\n"
            let normalString6 = NSMutableAttributedString(string: instText,
                                                          attributes: attrsNormal)
            var normalText2 = "Signal Strength Measured: " +
                beacon.getMssi().description + ", " +
                "Received: "
            var distLine = "Distance: "
            
            let rssi = data.rssi ?? 0
            if rssi != 0 {
                normalText2 += rssi.description  + " \n"
                distLine += String(format: "%.3f m\n",
                                   beacon.getDistance())
            } else {
                normalText2 += "Unknown \n"
                distLine += "Unknown \n"
            }
            let normalString2 = NSMutableAttributedString(string: normalText2,
                                                          attributes: attrsNormal)
            let normalString5 = NSMutableAttributedString(string: distLine,
                                                          attributes: attrsNormal)
            let temperature = (data.temperature ?? 0) / 256.0
            let tempLine = "Temperature: " +
                String(format: "%.3f C\n", temperature)
            let normalString3 = NSMutableAttributedString(string: tempLine,
                                                          attributes: attrsNormal)
            let voltage:Double = (data.voltage ?? 0)
            let voltLine = "Battery Voltage: " +
                String(format: "%.3f V", voltage/1000.0)
            let normalString4 = NSMutableAttributedString(string: voltLine,
                                                          attributes: attrsNormal)
            
            attributedString.append(normalString6)
            attributedString.append(normalString2)
            attributedString.append(normalString5)
            attributedString.append(normalString3)
            attributedString.append(normalString4)
            
            currentValueLabelOutlet.attributedText = attributedString
            break
        
        default:
            break
        }
        // Display the device updates.
        self.outputTextViewOutlet.text =
            self.outputTextViewOutlet.text + "\n" + selectedBeacon.getOutput()
        // Force text field to (invisibly) scroll until latest line is at bottom
        let bottom: NSRange = NSMakeRange(self.outputTextViewOutlet.text.count - 10, 1)
        self.outputTextViewOutlet.scrollRangeToVisible(bottom)
    }
}
