/*
 *  Copyright © 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

// The ApplicationSelectionView is used in conjunction with User Authentication
// but at this time it is not used.
// Code must be modified to use User Authentication.

// This screen is entered when the ServerLoginView successfully authenticates
// and displays this View.

import UIKit
import EnterpriseLib

class ApplicationSelectionView: UIViewController {

    @IBOutlet var applicationSelectionViewOutlet: UIView!
    @IBOutlet weak var iotApplicationLabelOutlet: UILabel!
    @IBOutlet weak var oracleLogoOutlet: UIImageView!
    @IBOutlet weak var applicationNameTextfieldOutlet: UITextField!
    
    // TODO: Use nav bar next button and back buttons
    @IBOutlet weak var nextButtonOutlet: UIButton!
    
    var applicationName:String = ""
    var enterpriseClientSample:EnterpriseClientSample? = nil
    var haveAuthError:Bool = false
    var ecready:Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.nextButtonOutlet.isEnabled = false
        self.title = NAVBARTITLE
    }

    override func viewWillAppear (_ animated: Bool) {
        self.navigationItem.setHidesBackButton(true, animated: false)
        
        // TODO: Eliminate globals
        if !self.applicationName.isEmpty {
            applicationNameTextfieldOutlet.text = self.applicationName
            self.nextButtonOutlet.isEnabled = true
        }
        // Set up notifications for terminate.
        // The app will shut down on either of these events.
        // This works in conjunction with the Info.plist property
        // Application does not run in background, or UIApplicationExitsOnSuspend.
        // When the "STOP" button is pressed, the app is put into suspend, and
        // because of the property "applicationWillTerminate" is called.
        NotificationCenter.default.addObserver(self, selector: #selector(self.terminate),
                                               name: UIApplication.willTerminateNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self)
        super.viewWillDisappear(animated)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func displayServerLoginView() {
         let serverLoginView = self.storyboard?.instantiateViewController(
            withIdentifier: "ServerLoginView") as! ServerLoginView
         self.navigationController?.pushViewController(serverLoginView,
         animated: false)
    }
    
    func displayDeviceModelSelectionView() {
        performSegue(withIdentifier: "AppSelViewToDevModelSelView", sender: applicationName)
    }
    
    func handleError(error:String) -> Bool {
        if isUserAuthenticationError(error:error) {
            self.enterpriseClientSample = nil
            displayServerLoginView()
            return true
        } else {
            print(error)
            // TODO: Exit ?
            showError(controller: self, alertTitle: "Enterprise Client Alert",
                      message: "\(error)", actionTitle: "OK",
                      handler: {(action:UIAlertAction?) -> () in
                        // When the app is suspended, because of the Info.plist property
                        // UIApplicationWillExitOnSuspend the UIApplicationWillTerminate
                        // observer is called and the "terminate" method will be called.
                        UIControl().sendAction(#selector(NSXPCConnection.suspend),
                                               to: UIApplication.shared, for: nil)
            })
            return false
        }
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        let dest:DeviceModelSelectionView = segue.destination as! DeviceModelSelectionView
        dest.applicationName = sender as? String
    }
    
    // actions
    // TODO: perform segue from the next button directly
    // Action for next button. The enteprise client is created, device model 
    // list and details are obtained for user authentication scheme.
    @IBAction func nextButtonOutletAction(_ sender: UIButton) {
        displayDeviceModelSelectionView()
    }
    
    @IBAction func onAppNameChanged(_ sender: UITextField) {
        self.nextButtonOutlet.isEnabled = true
        self.applicationName = applicationNameTextfieldOutlet.text!
    }
    
    @objc func terminate() {
        NotificationCenter.default.removeObserver(self)
        saveSettingsFile()
    }
    
}
