/*
 *  Copyright © 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */
import Foundation

class Provisioning {
    // Provisioning is static if there is a Truststore.plist
    // and it has information in it. If it is empty then
    // provisioning is not static.
    static var provStatic:Bool = { () -> Bool in
        var isStatic:Bool = false
        do {
            var settings:StaticSettings? = try loadBundledSettings()
            if settings == nil {
                settings = try loadLastSettings()
            } else {
                isStatic = true
            }
            if let settings = settings {
                Provisioning.setProvisioningFilePath(pf: settings.path)
                Provisioning.setProvisioningPassword(pp: settings.password)
            }
        } catch {
            provError = "\(error)"
        }
        return isStatic
    }()
    
    static var provError:String?
    class func getError() -> String? {
        return provError
    }
    class func clearError() {
        provError = nil
    }
    class func isProvisioned() -> Bool {
        return !provFilePath.isEmpty && !provPassword.isEmpty
    }
    
    class func isProvisioningStatic() -> Bool {
        return provStatic
    }
    
    // Make this true properties with getters and setters
    static var provFilePath:String = ""
    class func getProvisioningFilePath() -> String {
        return provFilePath
    }
    class func setProvisioningFilePath(pf: String) {
        provFilePath = pf
    }

    static var provPassword:String = ""
    class func getProvisioningPassword() -> String {
        return provPassword
    }
    class func setProvisioningPassword(pp: String) {
        provPassword = pp
    }

    
    // USE_POLICIES is defined in Constants.swift
    // Change that variable to change the default
    static var provWithPolicies:Bool = USE_POLICY
    class func getWithPolicies() -> Bool {
        return provWithPolicies
    }
    class func setWithPolicies(withPolicies:Bool) {
        provWithPolicies = withPolicies
    }
    
    static var reprovisioned:Bool = false
    class func isReprovisioned() -> Bool {
        return reprovisioned
    }
    class func reProvision() {
        removeLastSettingsFile()
        //removeFile(file: Provisioning.getProvisioningFilePath())
        Provisioning.setProvisioningFilePath(pf: "")
        Provisioning.setProvisioningPassword(pp: "")
        Provisioning.provStatic = false
        Provisioning.reprovisioned = true
    }
}
